----------------------------------------------------------------------------------
-- COPYRIGHT 2019 Jess Eduardo Mndez Rosales
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--
--							Convertidor de binario a decimal
--
-- Descripcin: Algoritmo Shift and Add 3 que convierte un valor binario en
-- 	vectores de 4 bits con valor decimal.
-- 
-- Caractersticas:
--
-- 1) El algoritmo procesa un vector de 27 bits y devuelve un vector de 32 bits
--	 con el valor decimal.
-- 2) Utiliza ciclos for para mejorar la velocidad de procesamiento
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_unsigned.ALL;

entity BINARIO_A_DECIMAL is

PORT(
		DATO_BINARIO : IN  STD_LOGIC_VECTOR(26 DOWNTO 0);
		DATO_DECIMAL : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)

);

end BINARIO_A_DECIMAL;

architecture Behavioral of BINARIO_A_DECIMAL is

begin

process(DATO_BINARIO)
variable z : std_logic_vector(59 downto 0);
begin
	for i in 0 to 59 loop
		z(i) := '0';
	end loop;
	z(29 downto 3) := DATO_BINARIO;
	
	for i in 0 to 23 loop
		if z(30 downto 27) > 4 then
			z(30 downto 27) := z(30 downto 27)+3;
		end if;
		
		if z(34 downto 31) > 4 then
			z(34 downto 31) := z(34 downto 31)+3;
		end if;
		
		if z(38 downto 35) > 4 then
			z(38 downto 35) := z(38 downto 35)+3;
		end if;
		
		if z(42 downto 39) > 4 then
			z(42 downto 39) := z(42 downto 39)+3;
		end if;
		
		if z(46 downto 43) > 4 then
			z(46 downto 43) := z(46 downto 43)+3;
		end if;
		
		if z(50 downto 47) > 4 then
			z(50 downto 47) := z(50 downto 47)+3;
		end if;
		
		if z(54 downto 51) > 4 then
			z(54 downto 51) := z(54 downto 51)+3;
		end if;
		
		if z(58 downto 55) > 4 then
			z(58 downto 55) := z(58 downto 55)+3;
		end if;
		
		z(59 downto 1) := z(58 downto 0);
		
	end loop;
	
	DATO_DECIMAL <= z(58 downto 27);
end process;

end Behavioral;

